# rejik_stat, v 0.9 (http://bio3k.softboard.ru/categories/3-wwwrejik)
# Copyright (C) 2005-2006 Elisey Savateev. All rights reserved.
#
# Redistribution and use in source form, with or without modification,
# are permitted provided that the following condition is met:
#    Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

#!/bin/sh

OPT_IP=0
OPT_IGN=0
USERS=""
TYPES=""

usage()
{
	echo "Usage: `basename "$0"` [-h | -i | -f] [-t <types>] [-u <users>] filename" >&2
	exit 1
}

help()
{
	cat << EOF
rejik_stat, v 0.9 (http://bio3k.softboard.ru/categories/3-wwwrejik)
Usage: `basename "$0"` [-h | -i | -f] [-t <types>] [-u <users>] filename
    -h Display this help message
    -i Use IP's instead of user names
    -f Ignore case distinctions in user names
    -t <types> Select only types witch specified in comma-separated list <types>
    -u <users> Select only users witch names (IP's) specified in comma-separated
       list <users>
Example: `basename "$0"` -t PORNO,MP3 -u Vasya redirector.log
EOF

	exit 0
}

if [ $# -lt 1 ]; then usage; fi

while getopts hift:u: OPT; do
	case $OPT in
		h)	help
			;;
		i)	OPT_IP=1
			;;
		f)	OPT_IGN=1
			;;
		t)	TYPES=`echo $OPTARG | tr ',' '\n'`
			;;
		u)	USERS=`echo $OPTARG | tr ',' '\n'`
			;;
		\?)	usage
			;;
	esac
done

if [ $# -gt 1 ]; then shift `expr $# - 1`; fi

if [ -f "$1" ]; then
	FROM=""
	TO=""

	FROM=`head -n 1 "$1" | grep -o '^[0-9-]\{10\} [0-9:]\{8\}'`
	TO=`tail -n 1 "$1" | grep -o '^[0-9-]\{10\} [0-9:]\{8\}'`

	if [ "$FROM" = "" -o "$TO" = "" ]; then
		echo "$1 does not contain needed information or empty." >&2
		exit 1
	fi
	
	echo -e "From $FROM to $TO\n"
	
	if [ $OPT_IP -eq 0 ]; then
		STAT=`cut -d ' ' -f 3,5 "$1"`
	else
		STAT=`cut -d ' ' -f 3,4 "$1"`
	fi

	if [ "$USERS" = "" ]; then 
		if [ $OPT_IGN -eq 0 ]; then
			USERS=`echo "$STAT" | cut -d ' ' -f 2 | sort -u`
		else
			USERS=`echo "$STAT" | cut -d ' ' -f 2 | sort -fu`
		fi
		if [ $OPT_IP -eq 0 -a "`echo "$USERS" | grep -x '-'`" != "" ]; then
			echo "$1 does not contain user names. Use '-i' option." >&2
			exit 1
		fi
	fi

	if [ "$TYPES" = "" ]; then
		TYPES=`echo "$STAT" | cut -d ' ' -f 1 | sort -u | tr -d ':'`
	fi

	echo -e "USER\t\t`echo "$TYPES" | tr '\n' '\t'`"
	echo "---------------------------------------------------------------------"

	for i in $USERS; do
		echo -e "$i\c"
		if [ ${#i} -lt 8 ]; then echo -e "\t\c"; fi
		
		for j in $TYPES; do
			if [ $OPT_IGN -eq 0 ]; then
				echo -e "\t`echo "$STAT" | grep -c "${j}: $i"`\c"
			else
				echo -e "\t`echo "$STAT" | grep -ic "${j}: $i"`\c"
			fi
		done

		echo
	done
else
	echo "No such file: $1" >&2
	exit 1
fi
